@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008, 2009, Yubico AB
+ * Copyright (c) 2008, 2009, 2010, Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,18 +62,6 @@ YK_KEY *yk_open_first_key(void)
 			rc = yk_errno;
 			yk_close_key(yk);
 			yk = NULL;
-		} else {
-			if (!((st.versionMajor == 1 &&
-			       (st.versionMinor == 0 ||
-				st.versionMinor == 1 ||
-				st.versionMinor == 2 ||
-				st.versionMinor == 3)) ||
-			      (st.versionMajor == 2 &&
-			       st.versionMinor == 0))) {
-				rc = YK_EFIRMWARE;
-				yk_close_key(yk);
-				yk = NULL;
-			}
 		}
 	}
 	yk_errno = rc;
@@ -85,6 +73,29 @@ int yk_close_key(YK_KEY *yk)
 	return _ykusb_close_device(yk);
 }
 
+int yk_check_firmware_version(YK_KEY *k)
+{
+	YK_STATUS st;
+
+	if (!yk_get_status(k, &st))
+		return 0;
+	if (!((st.versionMajor == 0 &&
+	       (st.versionMinor == 9 ||
+		st.versionBuild == 9)) ||
+	      (st.versionMajor == 1 &&
+	       (st.versionMinor == 0 ||
+		st.versionMinor == 1 ||
+		st.versionMinor == 2 ||
+		st.versionMinor == 3)) ||
+	      (st.versionMajor == 2 &&
+	       (st.versionMinor == 0 ||
+		st.versionMinor == 1)))) {
+		yk_errno = YK_EFIRMWARE;
+		return 0;
+	}
+	return 1;
+}
+
 int yk_get_status(YK_KEY *k, YK_STATUS *status)
 {
 	unsigned int status_count = 0;
